import math
import textwrap

from cpl_core.console import Console

from py_to_uxf_core.model.dimension import Dimension
from py_to_uxf_core.model.position import Position
from py_to_uxf_core.model.python_class import PythonClass


class UMLClass:

    def __init__(
            self,
            cls: PythonClass,
            pos: Position,
            dim: Dimension
    ):
        self._cls = cls
        self._position = pos
        self._dimension = dim

        self._attributes = ''
        self._functions = ''

    @property
    def cls(self) -> PythonClass:
        return self._cls

    @property
    def position(self) -> Position:
        return self._position

    @position.setter
    def position(self, value: int):
        self._position = value

    @property
    def dimension(self) -> Dimension:
        return self._dimension

    @dimension.setter
    def dimension(self, value: int):
        self._dimension = value

    def implementation_as_xml(self, base_class: 'UMLClass') -> str:
        base_point = Position(
            base_class.position.x + int(base_class.dimension.width / 2),
            base_class.position.y + base_class.dimension.height
        )

        sub_point = Position(
            self._position.x + int(self._dimension.width / 2),
            self._position.y
        )

        path_dimension = Dimension(
            self._position.x - base_class.position.x,
            sub_point.y - base_point.y
        )
        path_position = Position(
            0,
            0
        )

        return f"""\
            <element>
                <id>Relation</id>
                <coordinates>
                    <x>{base_point.x}</x>
                    <y>{base_point.y}</y>
                    <w>0</w>
                    <h>0</h>
                </coordinates>
                <panel_attributes>lt=&lt;&lt;-
                </panel_attributes>
                <additional_attributes>{float(path_position.x)};{float(path_position.y)};{float(path_dimension.width)};{float(path_dimension.height)}</additional_attributes>
            </element>
        """.replace('    ', '').replace('\t', '')

    def create_xml_body(self):
        px_per_line = 16
        px_per_char = 2.9
        self._dimension.height += int(
            round((self._cls.attributes.count() + self._cls.functions.count()) * px_per_line, -1))
        longest_line_length = self._dimension.width / px_per_char

        if len(self._cls.attributes) > 0:
            for atr in self._cls.attributes:
                attribute = f'{atr.access_modifier.value}{atr.name}: {atr.type}\n'
                if len(attribute) > longest_line_length:
                    longest_line_length = len(attribute)
                self._attributes += attribute

        if len(self._cls.functions) > 0:
            for func in self._cls.functions:
                args = ''
                function = f'{func.access_modifier.value}{func.name}({args}): {func.return_type}\n'
                if len(function) > longest_line_length:
                    longest_line_length = len(function)
                self._functions += function

        self._dimension.width = int(round(longest_line_length * px_per_char * px_per_char, -1))

    def as_xml(self) -> str:
        return f"""\
            <element>
                <id>UMLClass</id>
                <coordinates>
                    <x>{self._position.x}</x>
                    <y>{self._position.y}</y>
                    <w>{self._dimension.width}</w>
                    <h>{self._dimension.height}</h>
                </coordinates>
                <panel_attributes>
                    {self._cls.name}
                    --
                    {self._attributes}
                    --
                    {self._functions}
                </panel_attributes>
                <additional_attributes></additional_attributes>
            </element>
        """.replace('    ', '').replace('\t', '')
