from typing import Optional

from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console
from cpl_query.extension import List

from py_to_uxf_core.abc.implementation_scanner_abc import ImplementationScannerABC
from py_to_uxf_core.configuration.parser_settings import ParserSettings
from py_to_uxf_core.model.class_implementation import ClassImplementation
from py_to_uxf_core.model.python_class import PythonClass


class ImplementationScannerService(ImplementationScannerABC):

    def __init__(self, configuration: ConfigurationABC):
        ImplementationScannerABC.__init__(self)
        self._parser_settings: ParserSettings = configuration.get_configuration(ParserSettings)

    def scan_line_for_implementation(self, line: str, classes: List[PythonClass]) -> Optional[ClassImplementation]:
        line = line.replace('    ', '')
        line = line.replace('\t', '')
        if line.startswith('class ') and '(' in line and ')' in line:
            sub_name = line.split('class ')[1].split('(')[0]
            sub: Optional[PythonClass] = classes.where(lambda c: c.name == sub_name).first_or_default()
            if sub is None:
                return None

            base_name = line.split('(')[1].split(')')[0]
            base: Optional[PythonClass] = classes.where(lambda c: c.name == base_name).first_or_default()
            if base_name in self._parser_settings.ignore_class_names:
                return None

            if base is None:
                new_base = PythonClass(base_name)
                classes.append(new_base)
                return ClassImplementation(new_base, sub)

            return ClassImplementation(base, sub)
        return None
